import React$1 from 'react';

function _arrayLikeToArray(r, a) {
  (null == a || a > r.length) && (a = r.length);
  for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e];
  return n;
}
function _arrayWithoutHoles(r) {
  if (Array.isArray(r)) return _arrayLikeToArray(r);
}
function _iterableToArray(r) {
  if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r);
}
function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray(r) {
  return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread();
}
function _unsupportedIterableToArray(r, a) {
  if (r) {
    if ("string" == typeof r) return _arrayLikeToArray(r, a);
    var t = {}.toString.call(r).slice(8, -1);
    return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0;
  }
}

var findDifferences = function findDifferences(str1, str2) {
  var arr1 = str1.split('');
  var arr2 = str2.split('');
  var lcsMatrix = longestCommonSubsequence(arr1, arr2);
  var findResult = function findResult(i, j, result) {
    if (i > 0 && j > 0) {
      if (arr1[i - 1] === arr2[j - 1]) {
        return findResult(i - 1, j - 1, result);
      } else if (lcsMatrix[i - 1][j] > lcsMatrix[i][j - 1]) {
        return findResult(i - 1, j, [{
          type: 'removed',
          index: i - 1,
          "char": arr1[i - 1]
        }].concat(_toConsumableArray(result)));
      } else {
        return findResult(i, j - 1, [{
          type: 'added',
          index: j - 1,
          "char": arr2[j - 1]
        }].concat(_toConsumableArray(result)));
      }
    } else if (i > 0) {
      return findResult(i - 1, j, [{
        type: 'removed',
        index: i - 1,
        "char": arr1[i - 1]
      }].concat(_toConsumableArray(result)));
    } else if (j > 0) {
      return findResult(i, j - 1, [{
        type: 'added',
        index: j - 1,
        "char": arr2[j - 1]
      }].concat(_toConsumableArray(result)));
    } else {
      return result.reverse();
    }
  };
  return findResult(arr1.length, arr2.length, []);
};

var highlightDifferences = function highlightDifferences(str1, str2, highlightColor) {
  var differences = findDifferences(str1, str2);
  var highlightedStr1 = differences.reduce(function (acc, diff, index) {
    var prevIndex = index === 0 ? 0 : differences[index - 1].index + 1;
    if (diff.type === 'removed') {
      return [].concat(_toConsumableArray(acc), [/*#__PURE__*/React.createElement("span", {
        key: prevIndex
      }, str1.slice(prevIndex, diff.index)), /*#__PURE__*/React.createElement("span", {
        key: "r".concat(diff.index),
        style: {
          color: highlightColor
        }
      }, diff["char"])]);
    }
    return acc;
  }, []);
  var highlightedStr2 = differences.reduce(function (acc, diff, index) {
    var prevIndex = index === 0 ? 0 : differences[index - 1].index + 1;
    if (diff.type === 'added') {
      return [].concat(_toConsumableArray(acc), [/*#__PURE__*/React.createElement("span", {
        key: prevIndex
      }, str2.slice(prevIndex, diff.index)), /*#__PURE__*/React.createElement("span", {
        key: "a".concat(diff.index),
        style: {
          color: highlightColor
        }
      }, diff["char"])]);
    }
    return acc;
  }, []);
  var lastIndex1 = differences.length ? differences[differences.length - 1].index + 1 : 0;
  var lastIndex2 = differences.length ? differences[differences.length - 1].index + 1 : 0;
  highlightedStr1.push( /*#__PURE__*/React.createElement("span", {
    key: lastIndex1
  }, str1.slice(lastIndex1)));
  highlightedStr2.push( /*#__PURE__*/React.createElement("span", {
    key: lastIndex2
  }, str2.slice(lastIndex2)));
  return {
    highlightedStr1: highlightedStr1,
    highlightedStr2: highlightedStr2
  };
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".diff-container{background-color:grey;border:1px solid grey;display:flex;gap:16px;padding:16px}.diff-container.vertical{flex-direction:column}.diff-container.horizontal{flex-direction:row}.diff-box{background-color:#fff;flex:1;padding:16px}";
styleInject(css_248z);

var StringDiffHighlighter = function StringDiffHighlighter(_ref) {
  var str1 = _ref.str1,
    str2 = _ref.str2,
    _ref$highlightColor = _ref.highlightColor,
    highlightColor = _ref$highlightColor === void 0 ? 'yellow' : _ref$highlightColor,
    _ref$layout = _ref.layout,
    layout = _ref$layout === void 0 ? 'vertical' : _ref$layout;
  var _highlightDifferences = highlightDifferences(str1, str2, highlightColor),
    highlightedStr1 = _highlightDifferences.highlightedStr1,
    highlightedStr2 = _highlightDifferences.highlightedStr2;
  return /*#__PURE__*/React$1.createElement("div", {
    className: "diff-container ".concat(layout)
  }, /*#__PURE__*/React$1.createElement("div", {
    className: "diff-box"
  }, highlightedStr1), /*#__PURE__*/React$1.createElement("div", {
    className: "diff-box"
  }, highlightedStr2));
};

export { StringDiffHighlighter as default };
